/*
 * Projekt: C-quest
 * Autor: David Martinek (c) 2004
 * email: martinek@fit.vutbr.cz
 * Licence: GPL v.2
 * Popis: Zdrojov soubor s definic funkc pro prci se spojovmi seznamy.
 *
 * $Id: cqlist.c,v 1.8 2004/07/12 17:30:44 martinek Exp $
 * 
 * Copyright (C)2004 David Martinek
 *
 * Tento program je svobodn software; mete jej it a modifikovat podle
 * ustanoven GNU General Public License, vydvan Free Software
 * Foundation; a to bu verze 2 tto licence anebo (podle vaeho uven)
 * kterkoli pozdj verze.
 *
 * Tento program je roziovn v nadji, e bude uiten, avak BEZ
 * JAKKOLI ZRUKY; neposkytuj se ani odvozen zruky PRODEJNOSTI anebo
 * VHODNOSTI PRO URIT EL. Dal podrobnosti hledejte ve GNU General Public 
 * License.
 *
 * Kopii GNU General Public License jste ml obdret spolu s tmto
 * programem; pokud se tak nestalo, napite o ni Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "cqlist.h"
#include "cqtools.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>

/**
 * Inicializuje strukturu TItem.
 */
void initItem(TItem *item)
{
  assert(item != NULL);
  
  item->questLines = item->answLines = item->explLines = NULL;
  item->next = NULL;
  item->rightAnswer = 'a';
  item->time = 0;
}

/**
 * Vytvo a inicializuje strukturu TItem a vrt ukazatel na ni.
 */
TItem *createItem(void)
{
  TItem *item = myAlloc(sizeof(TItem));
  initItem(item);
  return item;
}

/**
 * Uvoln pam ze struktury TItem. Neuvoluje samotn parametr item, protoe
 * me jt o voln odkazem. Neuvoluje ani dal prvek spojovho seznamu.
 * Uvoluje pouze data.
 */
void freeItem(TItem *item)
{
  assert(item != NULL);
  // nen poteba provdt test, jestli promnn jsou rzn od NULL, protoe
  // funkce free si s tm porad sama
  free(item->questLines);
  free(item->answLines);
  free(item->explLines); 
  item->questLines = item->answLines = item->explLines = NULL;
}

/**
 * Inicializuje strukturu TTestData.
 */
void initTestData(TTestData *data)
{
  assert(data != NULL);
  
  data->title = NULL;
  data->totalTime = 0;
  data->total = 0;
  data->first = data->last = NULL;
}

/**
 * Vytvo a inicializuje strukturu TTestData a vrt ukazatel na ni.
 */
TTestData *createTestData(void)
{
  TTestData *data = myAlloc(sizeof(TTestData));
  initTestData(data);
  return data;
}

/**
 * Pid dal poloku seznamu otzek.
 */
void addItem(TTestData *data, TItem *item)
{
  assert(data != NULL && item != NULL);
  
  if (data->first == NULL)
  {
    data->first = data->last = item;
  }
  else
  {
    data->last->next = item;
    data->last = item;
  }

  data->total += 1;
}

/**
 * Sete asy vech poloek seznamu a ulo jej do sloky data->totalTime.
 */
void countTotalTime(TTestData *data)
{
  data->totalTime = 0;
  TItem *tmp = data->first;
  while (tmp != NULL)
  {
    data->totalTime += tmp->time;
    tmp = tmp->next;
  }
}

/**
 * Uvoln seznam testovch otzek. Neru parametr data, protoe me jt o
 * pedvn odkazem.
 */
void freeTestData(TTestData *data)
{
  assert(data != NULL);
  free(data->title); data->title = NULL;
  TItem *tmp;
  while ((tmp = data->first) != NULL)
  {
    data->first = tmp->next;
    freeItem(tmp);
    free(tmp);
  }
}

